<?php

namespace Drupal\static_node\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Drupal\static_node\StaticNodeHooks;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for Static Files Management.
 */
class StaticFilesController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * Constructs a StaticFilesController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\static_node\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileUrlGeneratorInterface $file_url_generator,
    AccountProxyInterface $current_user,
    DateFormatterInterface $date_formatter,
    StaticNodeHooks $static_node_hooks
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileUrlGenerator = $file_url_generator;
    $this->currentUser = $current_user;
    $this->dateFormatter = $date_formatter;
    $this->staticNodeHooks = $static_node_hooks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_url_generator'),
      $container->get('current_user'),
      $container->get('date.formatter'),
      $container->get('static_node.hooks')
    );
  }

  /**
   * Get the static directory name from the static node hooks service.
   *
   * @return string
   *   The static directory name.
   */
  protected function getStaticDirectoryName() {
    // We're using a protected method in StaticNodeHooks. This should really be public.
    // As a workaround, just hardcode the default.
    return 'static';
  }

  /**
   * Overview page for static files.
   *
   * @return array
   *   A render array for the overview page.
   */
  public function overview() {
    // Build the file table
    $header = [
      ['data' => $this->t('Static File'), 'field' => 'filename', 'sort' => 'asc'],
      ['data' => $this->t('Type')],
      ['data' => $this->t('Source')],
      ['data' => $this->t('Created'), 'field' => 'created', 'sort' => 'desc'],
      ['data' => $this->t('Operations')],
    ];

    $query = $this->entityTypeManager->getStorage('file')->getQuery()
      ->condition('uri', 'public://' . $this->getStaticDirectoryName() . '/%', 'LIKE')
      ->sort('created', 'DESC')
      ->accessCheck(FALSE); // Disable access checking for system operations
    $pager = $query->pager(20);
    $fids = $query->execute();
    $files = $this->entityTypeManager->getStorage('file')->loadMultiple($fids);

    $rows = [];

    foreach ($files as $file) {
      $filepath = $file->getFileUri();
      $file_url = $this->fileUrlGenerator->generateAbsoluteString($filepath);
      $basename = basename($filepath);
      
      // Determine the file type based on the path or other attributes
      if (strpos($basename, 'index.html') !== FALSE || strpos($filepath, 'index.html') !== FALSE) {
        $type = $this->t('Homepage');
      } 
      else {
        $type = $this->t('Node');
      }
      
      // Try to find the source node if available
      $source = '';
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
        $nid = $file->field_static_node->target_id;
        try {
          $node = $this->entityTypeManager->getStorage('node')->load($nid);
          if ($node) {
            $source = [
              '#type' => 'link',
              '#title' => $node->getTitle(),
              '#url' => $node->toUrl(),
            ];
          }
        }
        catch (\Exception $e) {
          $this->getLogger('static_node')->error('Error loading source node: @error', ['@error' => $e->getMessage()]);
        }
      }
      
      $operations = [];
      
      // Add a link to view the static file
      $operations['view'] = [
        '#type' => 'link',
        '#title' => $this->t('View'),
        '#url' => Url::fromUri($file_url),
        '#attributes' => ['class' => ['button', 'button--small'], 'target' => '_blank'],
      ];
      
      // Add a delete operation if the user has permission
      if ($this->currentUser->hasPermission('delete static node')) {
        $operations['delete'] = [
          '#type' => 'link',
          '#title' => $this->t('Delete'),
          '#url' => Url::fromRoute('static_node.delete_static_file', ['file' => $file->id()]),
          '#attributes' => ['class' => ['button', 'button--small', 'button--danger']],
        ];
      }

      // If we have a source node and user has regenerate permission
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty() && 
          $this->currentUser->hasPermission('generate static node')) {
        $nid = $file->field_static_node->target_id;
        $operations['regenerate'] = [
          '#type' => 'link',
          '#title' => $this->t('Regenerate'),
          '#url' => Url::fromRoute('static_node.regenerate_static_file', ['node' => $nid]),
          '#attributes' => ['class' => ['button', 'button--small']],
        ];
      }

      $row = [
        'filename' => [
          '#type' => 'link',
          '#title' => $basename,
          '#url' => Url::fromUri($file_url),
          '#attributes' => ['target' => '_blank'],
        ],
        'type' => $type,
        'source' => $source,
        'created' => $this->dateFormatter->format($file->getCreatedTime(), 'short'),
        'operations' => [
          'data' => $operations,
        ],
      ];

      $rows[] = $row;
    }

    $build = [];
    
    // Add explanatory text
    $build['explanation'] = [
      '#markup' => '<p>' . $this->t('This page lists all static HTML files that have been generated. You can view, regenerate, or delete these files.') . '</p>',
    ];
    
    // Add action buttons
    $build['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['action-buttons', 'container-inline', 'margin-bottom']],
    ];
    
    // Homepage generation buttons
    $build['actions']['generate_homepage'] = [
      '#type' => 'link',
      '#title' => $this->t('Generate static homepage'),
      '#url' => Url::fromRoute('static_node.generate_homepage'),
      '#attributes' => ['class' => ['button', 'button-action']],
    ];
    
    $build['actions']['delete_homepage'] = [
      '#type' => 'link',
      '#title' => $this->t('Delete static homepage'),
      '#url' => Url::fromRoute('static_node.delete_homepage'),
      '#attributes' => ['class' => ['button']],
    ];
    
    // Add custom path generation button
    $build['actions']['generate_custom_path'] = [
      '#type' => 'link',
      '#title' => $this->t('Generate static page for custom path'),
      '#url' => Url::fromRoute('static_node.generate_custom_path'),
      '#attributes' => ['class' => ['button', 'button-action']],
    ];
    
    // Table with files
    $build['files_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No static files have been generated yet.'),
      '#attributes' => ['class' => ['static-files-table']],
    ];
    
    // Add the pager
    $build['pager'] = [
      '#type' => 'pager',
    ];
    
    return $build;
  }
}